@@ -1,9 +1,13 @@
 /*************************************************
 * AlternativeName Source File                    *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #include <botan/asn1_obj.h>
+#include <botan/der_enc.h>
+#include <botan/ber_dec.h>
+#include <botan/oids.h>
+#include <botan/stl_util.h>
 #include <botan/charset.h>
 
 namespace Botan {
@@ -31,7 +35,7 @@ void AlternativeName::add_attribute(const std::string& type,
 
    typedef std::multimap<std::string, std::string>::iterator iter;
    std::pair<iter, iter> range = alt_info.equal_range(type);
-   for(iter j = range.first; j != range.second; j++)
+   for(iter j = range.first; j != range.second; ++j)
       if(j->second == str)
          return;
 
@@ -65,6 +69,24 @@ std::multimap<OID, ASN1_String> AlternativeName::get_othernames() const
    return othernames;
    }
 
+/*************************************************
+* Return all of the alternative names            *
+*************************************************/
+std::multimap<std::string, std::string> AlternativeName::contents() const
+   {
+   std::multimap<std::string, std::string> names;
+
+   typedef std::multimap<std::string, std::string>::const_iterator rdn_iter;
+   for(rdn_iter j = alt_info.begin(); j != alt_info.end(); ++j)
+      multimap_insert(names, j->first, j->second);
+
+   typedef std::multimap<OID, ASN1_String>::const_iterator on_iter;
+   for(on_iter j = othernames.begin(); j != othernames.end(); ++j)
+      multimap_insert(names, OIDS::lookup(j->first), j->second.value());
+
+   return names;
+   }
+
 /*************************************************
 * Return if this object has anything useful      *
 *************************************************/
@@ -73,62 +95,59 @@ bool AlternativeName::has_items() const
    return (alt_info.size() > 0 || othernames.size() > 0);
    }
 
-namespace DER {
+namespace {
 
 /*************************************************
-* DER encode a AlternativeName entry             *
+* DER encode an AlternativeName entry            *
 *************************************************/
-void encode_entries(DER_Encoder& encoder, const AlternativeName& alt_name,
+void encode_entries(DER_Encoder& encoder,
+                    const std::multimap<std::string, std::string>& attr,
                     const std::string& type, ASN1_Tag tagging)
    {
-   std::multimap<std::string, std::string> attr = alt_name.get_attributes();
-   typedef std::multimap<std::string, std::string>::iterator iter;
+   typedef std::multimap<std::string, std::string>::const_iterator iter;
 
    std::pair<iter, iter> range = attr.equal_range(type);
-   for(iter j = range.first; j != range.second; j++)
+   for(iter j = range.first; j != range.second; ++j)
       {
       ASN1_String asn1_string(j->second, IA5_STRING);
-      DER::encode(encoder, asn1_string, tagging, CONTEXT_SPECIFIC);
+      encoder.add_object(tagging, CONTEXT_SPECIFIC, asn1_string.iso_8859());
       }
    }
 
-/*************************************************
-* DER encode a AlternativeName                   *
-*************************************************/
-void encode(DER_Encoder& encoder, const AlternativeName& alt_name)
-   {
-   encoder.start_sequence();
-
-   encode_entries(encoder, alt_name, "RFC822", ASN1_Tag(1));
-   encode_entries(encoder, alt_name, "DNS", ASN1_Tag(2));
-   encode_entries(encoder, alt_name, "URI", ASN1_Tag(6));
-
-   std::multimap<OID, ASN1_String> othernames = alt_name.get_othernames();
-
-   std::multimap<OID, ASN1_String>::const_iterator i;
-   for(i = othernames.begin(); i != othernames.end(); i++)
-      {
-      encoder.start_explicit(ASN1_Tag(0));
-      DER::encode(encoder, i->first);
-         encoder.start_explicit(ASN1_Tag(0));
-            DER::encode(encoder, i->second);
-         encoder.end_explicit(ASN1_Tag(0));
-      encoder.end_explicit(ASN1_Tag(0));
-      }
-
-   encoder.end_sequence();
-   }
-
 }
 
-namespace BER {
+/*************************************************
+* DER encode an AlternativeName extension        *
+*************************************************/
+void AlternativeName::encode_into(DER_Encoder& der) const
+   {
+   der.start_cons(SEQUENCE);
+
+   encode_entries(der, alt_info, "RFC822", ASN1_Tag(1));
+   encode_entries(der, alt_info, "DNS", ASN1_Tag(2));
+   encode_entries(der, alt_info, "URI", ASN1_Tag(6));
+
+   std::multimap<OID, ASN1_String>::const_iterator i;
+   for(i = othernames.begin(); i != othernames.end(); ++i)
+      {
+      der.start_explicit(0)
+         .encode(i->first)
+         .start_explicit(0)
+            .encode(i->second)
+         .end_explicit()
+      .end_explicit();
+      }
+
+   der.end_cons();
+   }
 
 /*************************************************
 * Decode a BER encoded AlternativeName           *
 *************************************************/
-void decode(BER_Decoder& source, AlternativeName& alt_name)
+void AlternativeName::decode_from(BER_Decoder& source)
    {
-   BER_Decoder names = BER::get_subsequence(source);
+   BER_Decoder names = source.start_cons(SEQUENCE);
+
    while(names.more_items())
       {
       BER_Object obj = names.get_next_object();
@@ -143,7 +162,7 @@ void decode(BER_Decoder& source, AlternativeName& alt_name)
          BER_Decoder othername(obj.value);
 
          OID oid;
-         BER::decode(othername, oid);
+         othername.decode(oid);
          if(othername.more_items())
             {
             BER_Object othername_value_outer = othername.get_next_object();
@@ -163,21 +182,21 @@ void decode(BER_Decoder& source, AlternativeName& alt_name)
             ASN1_Tag value_type = value.type_tag;
 
             if(is_string_type(value_type) && value.class_tag == UNIVERSAL)
-               alt_name.add_othername(oid, BER::to_string(value), value_type);
+               add_othername(oid, ASN1::to_string(value), value_type);
             }
          }
       else if(tag == 1 || tag == 2 || tag == 6)
          {
-         const std::string value = iso2local(BER::to_string(obj));
+         const std::string value = Charset::transcode(ASN1::to_string(obj),
+                                                      LATIN1_CHARSET,
+                                                      LOCAL_CHARSET);
 
-         if(tag == 1) alt_name.add_attribute("RFC822", value);
-         if(tag == 2) alt_name.add_attribute("DNS", value);
-         if(tag == 6) alt_name.add_attribute("URI", value);
+         if(tag == 1) add_attribute("RFC822", value);
+         if(tag == 2) add_attribute("DNS", value);
+         if(tag == 6) add_attribute("URI", value);
          }
 
       }
    }
 
 }
-
-}
