@@ -1,11 +1,12 @@
 /*************************************************
 * CTR Mode Source File                           *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #include <botan/ctr.h>
 #include <botan/lookup.h>
 #include <botan/bit_ops.h>
+#include <algorithm>
 
 namespace Botan {
 
@@ -63,7 +64,7 @@ void CTR_BE::write(const byte input[], u32bit length)
 *************************************************/
 void CTR_BE::increment_counter()
    {
-   for(s32bit j = BLOCK_SIZE - 1; j >= 0; j--)
+   for(s32bit j = BLOCK_SIZE - 1; j >= 0; --j)
       if(++state[j])
          break;
    cipher->encrypt(state, buffer);
