@@ -1,17 +1,18 @@
 /*************************************************
 * Engine Header File                             *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #ifndef BOTAN_ENGINE_H__
 #define BOTAN_ENGINE_H__
 
 #include <botan/base.h>
-#include <botan/pk_ops.h>
-#include <botan/pkcs8.h>
-#include <botan/reducer.h>
-#include <botan/basefilt.h>
 #include <botan/mutex.h>
+#include <botan/pk_ops.h>
+#include <botan/pow_mod.h>
+#include <botan/basefilt.h>
+#include <botan/enums.h>
+#include <utility>
 #include <map>
 
 namespace Botan {
@@ -22,6 +23,15 @@ namespace Botan {
 class Engine
    {
    public:
+      template<typename T>
+      class Algorithm_Cache
+         {
+         public:
+            virtual T* get(const std::string&) const = 0;
+            virtual void add(T* algo) const = 0;
+            virtual ~Algorithm_Cache() {}
+         };
+
       virtual IF_Operation* if_op(const BigInt&, const BigInt&, const BigInt&,
                                   const BigInt&, const BigInt&, const BigInt&,
                                   const BigInt&, const BigInt&) const;
@@ -32,19 +42,26 @@ class Engine
       virtual ELG_Operation* elg_op(const DL_Group&, const BigInt&,
                                     const BigInt&) const;
       virtual DH_Operation* dh_op(const DL_Group&, const BigInt&) const;
-      virtual ModularReducer* reducer(const BigInt&, bool) const;
+
+      virtual Modular_Exponentiator* mod_exp(const BigInt&,
+                                             Power_Mod::Usage_Hints) const;
+
+      virtual Keyed_Filter* get_cipher(const std::string&, Cipher_Dir);
 
       const BlockCipher* block_cipher(const std::string&) const;
       const StreamCipher* stream_cipher(const std::string&) const;
       const HashFunction* hash(const std::string&) const;
       const MessageAuthenticationCode* mac(const std::string&) const;
-
-      virtual Keyed_Filter* get_cipher(const std::string&, Cipher_Dir);
+      const class S2K* s2k(const std::string&) const;
+      const class BlockCipherModePaddingMethod*
+         bc_pad(const std::string&) const;
 
       void add_algorithm(BlockCipher*) const;
       void add_algorithm(StreamCipher*) const;
       void add_algorithm(HashFunction*) const;
       void add_algorithm(MessageAuthenticationCode*) const;
+      void add_algorithm(class S2K*) const;
+      void add_algorithm(class BlockCipherModePaddingMethod*) const;
 
       Engine();
       virtual ~Engine();
@@ -53,28 +70,37 @@ class Engine
       virtual StreamCipher* find_stream_cipher(const std::string&) const;
       virtual HashFunction* find_hash(const std::string&) const;
       virtual MessageAuthenticationCode* find_mac(const std::string&) const;
+      virtual class S2K* find_s2k(const std::string&) const;
+      virtual class BlockCipherModePaddingMethod*
+         find_bc_pad(const std::string&) const;
 
-      mutable std::map<std::string, BlockCipher*> bc_map;
-      mutable std::map<std::string, StreamCipher*> sc_map;
-      mutable std::map<std::string, HashFunction*> hf_map;
-      mutable std::map<std::string, MessageAuthenticationCode*> mac_map;
+      template<typename T>
+      T* lookup_algo(const Algorithm_Cache<T>* cache,
+                     const std::string& name,
+                     const Engine* engine,
+                     T* (Engine::*find)(const std::string& name) const) const
+         {
+         T* algo = cache->get(name);
+         if(!algo)
+            cache->add(algo = (engine->*find)(name));
+         return algo;
+         }
 
-      Mutex* bc_map_lock;
-      Mutex* sc_map_lock;
-      Mutex* hf_map_lock;
-      Mutex* mac_map_lock;
+      Algorithm_Cache<BlockCipher>* cache_of_bc;
+      Algorithm_Cache<StreamCipher>* cache_of_sc;
+      Algorithm_Cache<HashFunction>* cache_of_hf;
+      Algorithm_Cache<MessageAuthenticationCode>* cache_of_mac;
+      Algorithm_Cache<BlockCipherModePaddingMethod>* cache_of_bc_pad;
+      Algorithm_Cache<S2K>* cache_of_s2k;
    };
 
 namespace Engine_Core {
 
-/*************************************************
-* Engine Management                              *
-*************************************************/
-void add_engine(Engine*);
-
 /*************************************************
 * Get an operation from an Engine                *
 *************************************************/
+Modular_Exponentiator* mod_exp(const BigInt&, Power_Mod::Usage_Hints);
+
 IF_Operation* if_op(const BigInt&, const BigInt&, const BigInt&,
                     const BigInt&, const BigInt&, const BigInt&,
                     const BigInt&, const BigInt&);
