@@ -1,6 +1,6 @@
 /*************************************************
 * Number Theory Header File                      *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #ifndef BOTAN_NUMBTHRY_H__
@@ -8,6 +8,7 @@
 
 #include <botan/bigint.h>
 #include <botan/reducer.h>
+#include <botan/pow_mod.h>
 
 namespace Botan {
 
@@ -16,7 +17,6 @@ namespace Botan {
 *************************************************/
 BigInt mul_add(const BigInt&, const BigInt&, const BigInt&);
 BigInt sub_mul(const BigInt&, const BigInt&, const BigInt&);
-BigInt mul_mod(const BigInt&, const BigInt&, const BigInt&);
 
 /*************************************************
 * Number Theory Functions                        *
@@ -24,8 +24,6 @@ BigInt mul_mod(const BigInt&, const BigInt&, const BigInt&);
 inline BigInt abs(const BigInt& n) { return n.abs(); }
 
 void divide(const BigInt&, const BigInt&, BigInt&, BigInt&);
-void positive_divide(const BigInt&, const BigInt&, BigInt&, BigInt&);
-void modifying_divide(BigInt&, BigInt&, BigInt&);
 
 BigInt gcd(const BigInt&, const BigInt&);
 BigInt lcm(const BigInt&, const BigInt&);
@@ -33,12 +31,7 @@ BigInt lcm(const BigInt&, const BigInt&);
 BigInt square(const BigInt&);
 BigInt inverse_mod(const BigInt&, const BigInt&);
 s32bit jacobi(const BigInt&, const BigInt&);
-BigInt power(const BigInt&, u32bit);
 
-/*************************************************
-* Modular Exponentiation                         *
-*************************************************/
-BigInt power_mod(const BigInt&, const BigInt&, ModularReducer*);
 BigInt power_mod(const BigInt&, const BigInt&, const BigInt&);
 
 /*************************************************
@@ -60,11 +53,10 @@ bool run_primality_tests(const BigInt&, u32bit = 1);
 /*************************************************
 * Random Number Generation                       *
 *************************************************/
-BigInt random_integer(u32bit, RNG_Quality = SessionKey);
-BigInt random_integer(const BigInt&, const BigInt&, RNG_Quality = SessionKey);
-BigInt random_prime(u32bit, RNG_Quality = SessionKey, const BigInt& = 1,
-                    u32bit = 1, u32bit = 2);
-BigInt random_safe_prime(u32bit, RNG_Quality = SessionKey);
+BigInt random_integer(u32bit);
+BigInt random_integer(const BigInt&, const BigInt&);
+BigInt random_prime(u32bit, const BigInt& = 1, u32bit = 1, u32bit = 2);
+BigInt random_safe_prime(u32bit);
 
 SecureVector<byte> generate_dsa_primes(BigInt&, BigInt&, u32bit);
 bool generate_dsa_primes(BigInt&, BigInt&, const byte[], u32bit, u32bit,
@@ -86,15 +78,12 @@ class MillerRabin_Test
    {
    public:
       bool passes_test(const BigInt&);
-
       MillerRabin_Test(const BigInt&);
-      ~MillerRabin_Test() { delete reducer; }
    private:
-      MillerRabin_Test(const MillerRabin_Test&) {}
-      MillerRabin_Test& operator=(const MillerRabin_Test&) { return (*this); }
       BigInt n, r, n_minus_1;
       u32bit s;
-      ModularReducer* reducer;
+      Fixed_Exponent_Power_Mod pow_mod;
+      Modular_Reducer reducer;
    };
 
 }
