@@ -1,6 +1,6 @@
 /*************************************************
 * PKCS #10 Header File                           *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #ifndef BOTAN_PKCS10_H__
@@ -8,6 +8,7 @@
 
 #include <botan/x509_obj.h>
 #include <botan/pkcs8.h>
+#include <botan/datastor.h>
 #include <vector>
 
 namespace Botan {
@@ -18,7 +19,7 @@ namespace Botan {
 class PKCS10_Request : public X509_Object
    {
    public:
-      X509_PublicKey* subject_public_key() const;
+      Public_Key* subject_public_key() const;
 
       MemoryVector<byte> raw_public_key() const;
       X509_DN subject_dn() const;
@@ -36,16 +37,8 @@ class PKCS10_Request : public X509_Object
    private:
       void force_decode();
       void handle_attribute(const Attribute&);
-      void handle_v3_extension(const Extension&);
 
-      MemoryVector<byte> pub_key;
-      X509_DN dn;
-      AlternativeName subject_alt;
-      std::string challenge;
-      Key_Constraints constraints_value;
-      std::vector<OID> ex_constraints_list;
-      bool is_ca;
-      u32bit max_path_len;
+      Data_Store info;
    };
 
 }
