@@ -1,6 +1,6 @@
 /*************************************************
 * X9.42 PRF Source File                          *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #include <botan/kdf.h>
@@ -8,6 +8,7 @@
 #include <botan/oids.h>
 #include <botan/lookup.h>
 #include <botan/bit_ops.h>
+#include <algorithm>
 #include <memory>
 
 namespace Botan {
@@ -19,13 +20,11 @@ namespace {
 *************************************************/
 MemoryVector<byte> encode_x942_int(u32bit n)
    {
-   byte n_buf[4];
-   for(u32bit j = 0; j != 4; j++)
+   byte n_buf[4] = { 0 };
+   for(u32bit j = 0; j != 4; ++j)
       n_buf[j] = get_byte(j, n);
 
-   DER_Encoder encoder;
-   DER::encode(encoder, n_buf, 4, OCTET_STRING);
-   return encoder.get_contents();
+   return DER_Encoder().encode(n_buf, 4, OCTET_STRING).get_contents();
    }
 
 }
@@ -45,31 +44,34 @@ SecureVector<byte> X942_PRF::derive(u32bit key_len,
 
    while(key.size() != key_len)
       {
-      DER_Encoder encoder;
-      encoder.start_sequence();
-        encoder.start_sequence();
-          DER::encode(encoder, kek_algo);
-          encoder.add_raw_octets(encode_x942_int(counter));
-        encoder.end_sequence();
-
-        if(salt_len)
-           {
-           encoder.start_explicit(ASN1_Tag(0));
-           DER::encode(encoder, salt, salt_len, OCTET_STRING);
-           encoder.end_explicit(ASN1_Tag(0));
-           }
-
-        encoder.start_explicit(ASN1_Tag(2));
-          encoder.add_raw_octets(encode_x942_int(8 * key_len));
-        encoder.end_explicit(ASN1_Tag(2));
-      encoder.end_sequence();
-
       hash->update(secret, secret_len);
-      hash->update(encoder.get_contents());
+
+      hash->update(
+         DER_Encoder().start_cons(SEQUENCE)
+
+            .start_cons(SEQUENCE)
+               .encode(kek_algo)
+               .raw_bytes(encode_x942_int(counter))
+            .end_cons()
+
+            .encode_if(salt_len != 0,
+               DER_Encoder()
+                  .start_explicit(0)
+                     .encode(salt, salt_len, OCTET_STRING)
+                  .end_explicit()
+               )
+
+            .start_explicit(2)
+               .raw_bytes(encode_x942_int(8 * key_len))
+            .end_explicit()
+
+         .end_cons().get_contents()
+         );
+
       SecureVector<byte> digest = hash->final();
       key.append(digest, std::min(digest.size(), key_len - key.size()));
 
-      counter++;
+      ++counter;
       }
 
    return key;
