@@ -1,6 +1,6 @@
 /*************************************************
 * RSA Source File                                *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #include <botan/rsa.h>
@@ -60,12 +60,11 @@ RSA_PrivateKey::RSA_PrivateKey(u32bit bits, u32bit exp)
       throw Invalid_Argument(algo_name() + ": Invalid encryption exponent");
 
    e = exp;
-   p = random_prime((bits + 1) / 2, LongTermKey, e);
-   q = random_prime(bits - p.bits(), LongTermKey, e);
+   p = random_prime((bits + 1) / 2, e);
+   q = random_prime(bits - p.bits(), e);
    d = inverse_mod(e, lcm(p - 1, q - 1));
 
-   PKCS8_load_hook();
-   check_generated_private();
+   PKCS8_load_hook(true);
 
    if(n.bits() != bits)
       throw Self_Test_Failure(algo_name() + " private key generation failed");
@@ -88,7 +87,6 @@ RSA_PrivateKey::RSA_PrivateKey(const BigInt& prime1, const BigInt& prime2,
       d = inverse_mod(e, lcm(p - 1, q - 1));
 
    PKCS8_load_hook();
-   check_loaded_private();
    }
 
 /*************************************************
