@@ -1,6 +1,6 @@
 /*************************************************
 * SHA-160 Header File                            *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #ifndef BOTAN_SHA_160_H__
@@ -19,16 +19,13 @@ class SHA_160 : public MDx_HashFunction
       void clear() throw();
       std::string name() const { return "SHA-160"; }
       HashFunction* clone() const { return new SHA_160; }
-      SHA_160() : MDx_HashFunction(20, 64, true, true) { clear(); }
+      SHA_160();
    private:
-      friend class Gamma;
-      friend class FIPS_186_RNG;
-
       void hash(const byte[]);
       void copy_out(byte[]);
 
       SecureBuffer<u32bit, 5> digest;
-      SecureBuffer<u32bit, 80> W;
+      SecureVector<u32bit> W;
    };
 
 }
