@@ -1,11 +1,18 @@
 /*************************************************
 * X.509 SIGNED Object Source File                *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #include <botan/x509_obj.h>
+#include <botan/x509_key.h>
+#include <botan/look_pk.h>
+#include <botan/oids.h>
+#include <botan/der_enc.h>
+#include <botan/ber_dec.h>
 #include <botan/parsing.h>
 #include <botan/pem.h>
+#include <algorithm>
+#include <memory>
 
 namespace Botan {
 
@@ -39,7 +46,7 @@ void X509_Object::init(DataSource& in, const std::string& labels)
    std::sort(PEM_labels_allowed.begin(), PEM_labels_allowed.end());
 
    try {
-      if(BER::maybe_BER(in) && !PEM_Code::matches(in))
+      if(ASN1::maybe_BER(in) && !PEM_Code::matches(in))
          decode_info(in);
       else
          {
@@ -63,13 +70,15 @@ void X509_Object::init(DataSource& in, const std::string& labels)
 *************************************************/
 void X509_Object::decode_info(DataSource& source)
    {
-   BER_Decoder ber(source);
-   BER_Decoder sequence = BER::get_subsequence(ber);
-   tbs_bits = BER::get_subsequence(sequence).get_remaining();
-
-   BER::decode(sequence, sig_algo);
-   BER::decode(sequence, sig, BIT_STRING);
-   sequence.verify_end();
+   BER_Decoder(source)
+      .start_cons(SEQUENCE)
+         .start_cons(SEQUENCE)
+            .raw_bytes(tbs_bits)
+         .end_cons()
+         .decode(sig_algo)
+         .decode(sig, BIT_STRING)
+         .verify_end()
+      .end_cons();
    }
 
 /*************************************************
@@ -77,15 +86,16 @@ void X509_Object::decode_info(DataSource& source)
 *************************************************/
 void X509_Object::encode(Pipe& out, X509_Encoding encoding) const
    {
-   DER_Encoder encoder;
+   SecureVector<byte> der = DER_Encoder()
+      .start_cons(SEQUENCE)
+         .start_cons(SEQUENCE)
+            .raw_bytes(tbs_bits)
+         .end_cons()
+         .encode(sig_algo)
+         .encode(sig, BIT_STRING)
+      .end_cons()
+   .get_contents();
 
-   encoder.start_sequence();
-   encoder.add_raw_octets(tbs_data());
-   DER::encode(encoder, sig_algo);
-   DER::encode(encoder, sig, BIT_STRING);
-   encoder.end_sequence();
-
-   SecureVector<byte> der = encoder.get_contents();
    if(encoding == PEM)
       out.write(PEM_Code::encode(der, PEM_label_pref));
    else
@@ -121,7 +131,7 @@ std::string X509_Object::PEM_encode() const
 *************************************************/
 SecureVector<byte> X509_Object::tbs_data() const
    {
-   return DER::put_in_sequence(tbs_bits);
+   return ASN1::put_in_sequence(tbs_bits);
    }
 
 /*************************************************
@@ -140,6 +150,63 @@ AlgorithmIdentifier X509_Object::signature_algorithm() const
    return sig_algo;
    }
 
+/*************************************************
+* Check the signature on an object               *
+*************************************************/
+bool X509_Object::check_signature(Public_Key& pub_key) const
+   {
+   try {
+      std::vector<std::string> sig_info =
+         split_on(OIDS::lookup(sig_algo.oid), '/');
+
+      if(sig_info.size() != 2 || sig_info[0] != pub_key.algo_name())
+         return false;
+
+      std::string padding = sig_info[1];
+      Signature_Format format =
+         (pub_key.message_parts() >= 2) ? DER_SEQUENCE : IEEE_1363;
+
+      std::auto_ptr<PK_Verifier> verifier;
+
+      if(dynamic_cast<PK_Verifying_with_MR_Key*>(&pub_key))
+         {
+         PK_Verifying_with_MR_Key& sig_key =
+            dynamic_cast<PK_Verifying_with_MR_Key&>(pub_key);
+         verifier.reset(get_pk_verifier(sig_key, padding, format));
+         }
+      else if(dynamic_cast<PK_Verifying_wo_MR_Key*>(&pub_key))
+         {
+         PK_Verifying_wo_MR_Key& sig_key =
+            dynamic_cast<PK_Verifying_wo_MR_Key&>(pub_key);
+         verifier.reset(get_pk_verifier(sig_key, padding, format));
+         }
+      else
+         return false;
+
+      return verifier->verify_message(tbs_data(), signature());
+      }
+   catch(...)
+      {
+      return false;
+      }
+   }
+
+/*************************************************
+* Apply the X.509 SIGNED macro                   *
+*************************************************/
+MemoryVector<byte> X509_Object::make_signed(PK_Signer* signer,
+                                            const AlgorithmIdentifier& algo,
+                                            const MemoryRegion<byte>& tbs_bits)
+   {
+   return DER_Encoder()
+      .start_cons(SEQUENCE)
+         .raw_bytes(tbs_bits)
+         .encode(algo)
+         .encode(signer->sign_message(tbs_bits), BIT_STRING)
+      .end_cons()
+   .get_contents();
+   }
+
 /*************************************************
 * Try to decode the actual information           *
 *************************************************/
