@@ -7,6 +7,7 @@
 // implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 // PURPOSE.
 
+#include "base.hh"
 #include <deque>
 #include <map>
 #include <iostream>
@@ -92,15 +93,15 @@ changes_summary::add_change_set(cset const & c)
 
 static void
 print_indented_set(ostream & os,
-                   path_set const & s,
+                   set<file_path> const & s,
                    size_t max_cols)
 {
   size_t cols = 8;
   os << "       ";
-  for (path_set::const_iterator i = s.begin();
+  for (set<file_path>::const_iterator i = s.begin();
        i != s.end(); i++)
     {
-      const string str = lexical_cast<string>(file_path(*i));
+      const string str = lexical_cast<string>(*i);
       if (cols > 8 && cols + str.size() + 1 >= max_cols)
         {
           cols = 8;
@@ -125,17 +126,17 @@ changes_summary::print(ostream & os, size_t max_cols) const
   if (! cs.nodes_renamed.empty())
     {
       os << _("Renamed entries:") << '\n';
-      for (map<split_path, split_path>::const_iterator
+      for (map<file_path, file_path>::const_iterator
            i = cs.nodes_renamed.begin();
            i != cs.nodes_renamed.end(); i++)
-        os << "        " << file_path(i->first)
-           << " to " << file_path(i->second) << '\n';
+        os << "        " << i->first
+           << " to " << i->second << '\n';
     }
 
   if (! cs.files_added.empty())
     {
-      path_set tmp;
-      for (map<split_path, file_id>::const_iterator
+      set<file_path> tmp;
+      for (map<file_path, file_id>::const_iterator
              i = cs.files_added.begin();
            i != cs.files_added.end(); ++i)
         tmp.insert(i->first);
@@ -151,8 +152,8 @@ changes_summary::print(ostream & os, size_t max_cols) const
 
   if (! cs.deltas_applied.empty())
     {
-      path_set tmp;
-      for (map<split_path, pair<file_id, file_id> >::const_iterator
+      set<file_path> tmp;
+      for (map<file_path, pair<file_id, file_id> >::const_iterator
              i = cs.deltas_applied.begin();
            i != cs.deltas_applied.end(); ++i)
         tmp.insert(i->first);
@@ -162,13 +163,13 @@ changes_summary::print(ostream & os, size_t max_cols) const
 
   if (! cs.attrs_set.empty() || ! cs.attrs_cleared.empty())
     {
-      path_set tmp;
-      for (set<pair<split_path, attr_key> >::const_iterator
+      set<file_path> tmp;
+      for (set<pair<file_path, attr_key> >::const_iterator
              i = cs.attrs_cleared.begin();
            i != cs.attrs_cleared.end(); ++i)
         tmp.insert(i->first);
 
-      for (map<pair<split_path, attr_key>, attr_value>::const_iterator
+      for (map<pair<file_path, attr_key>, attr_value>::const_iterator
              i = cs.attrs_set.begin();
            i != cs.attrs_set.end(); ++i)
         tmp.insert(i->first.first);
@@ -183,7 +184,7 @@ do_external_diff(cset const & cs,
                  app_state & app,
                  bool new_is_archived)
 {
-  for (map<split_path, pair<file_id, file_id> >::const_iterator
+  for (map<file_path, pair<file_id, file_id> >::const_iterator
          i = cs.deltas_applied.begin();
        i != cs.deltas_applied.end(); ++i)
     {
@@ -202,7 +203,7 @@ do_external_diff(cset const & cs,
         }
       else
         {
-          read_data(file_path(delta_entry_path(i)), data_new);
+          read_data(delta_entry_path(i), data_new);
         }
 
       bool is_binary = false;
@@ -210,7 +211,7 @@ do_external_diff(cset const & cs,
           guess_binary(data_new()))
         is_binary = true;
 
-      app.lua.hook_external_diff(file_path(delta_entry_path(i)),
+      app.lua.hook_external_diff(delta_entry_path(i),
                                  data_old,
                                  data_new,
                                  is_binary,
@@ -226,13 +227,13 @@ dump_diffs(cset const & cs,
            app_state & app,
            bool new_is_archived,
            std::ostream & output,
-           set<split_path> const & paths,
+           set<file_path> const & paths,
            bool limit_paths = false)
 {
   // 60 is somewhat arbitrary, but less than 80
   string patch_sep = string(60, '=');
 
-  for (map<split_path, file_id>::const_iterator
+  for (map<file_path, file_id>::const_iterator
          i = cs.files_added.begin();
        i != cs.files_added.end(); ++i)
     {
@@ -251,32 +252,31 @@ dump_diffs(cset const & cs,
         }
       else
         {
-          read_data(file_path(i->first), unpacked);
+          read_data(i->first, unpacked);
         }
 
       std::string pattern("");
       if (!app.opts.no_show_encloser)
-        app.lua.hook_get_encloser_pattern(file_path(i->first),
-                                          pattern);
+        app.lua.hook_get_encloser_pattern(i->first, pattern);
 
-      make_diff(file_path(i->first).as_internal(),
-                file_path(i->first).as_internal(),
+      make_diff(i->first.as_internal(),
+                i->first.as_internal(),
                 i->second,
                 i->second,
                 data(), unpacked,
                 output, app.opts.diff_format, pattern);
     }
 
-  map<split_path, split_path> reverse_rename_map;
+  map<file_path, file_path> reverse_rename_map;
 
-  for (map<split_path, split_path>::const_iterator
+  for (map<file_path, file_path>::const_iterator
          i = cs.nodes_renamed.begin();
        i != cs.nodes_renamed.end(); ++i)
     {
       reverse_rename_map.insert(make_pair(i->second, i->first));
     }
 
-  for (map<split_path, pair<file_id, file_id> >::const_iterator
+  for (map<file_path, pair<file_id, file_id> >::const_iterator
          i = cs.deltas_applied.begin();
        i != cs.deltas_applied.end(); ++i)
     {
@@ -299,23 +299,23 @@ dump_diffs(cset const & cs,
         }
       else
         {
-          read_data(file_path(delta_entry_path(i)), data_new);
+          read_data(delta_entry_path(i), data_new);
         }
 
-      split_path dst_path = delta_entry_path(i);
-      split_path src_path = dst_path;
-      map<split_path, split_path>::const_iterator re;
+      file_path dst_path = delta_entry_path(i);
+      file_path src_path = dst_path;
+      map<file_path, file_path>::const_iterator re;
       re = reverse_rename_map.find(dst_path);
       if (re != reverse_rename_map.end())
         src_path = re->second;
 
       std::string pattern("");
       if (!app.opts.no_show_encloser)
-        app.lua.hook_get_encloser_pattern(file_path(src_path),
+        app.lua.hook_get_encloser_pattern(src_path,
                                           pattern);
 
-      make_diff(file_path(src_path).as_internal(),
-                file_path(dst_path).as_internal(),
+      make_diff(src_path.as_internal(),
+                dst_path.as_internal(),
                 delta_entry_src(i),
                 delta_entry_dst(i),
                 data_old, data_new,
@@ -329,7 +329,7 @@ dump_diffs(cset const & cs,
            bool new_is_archived,
            std::ostream & output)
 {
-  set<split_path> dummy;
+  set<file_path> dummy;
   dump_diffs(cs, app, new_is_archived, output, dummy);
 }
 
@@ -338,7 +338,7 @@ dump_diffs(cset const & cs,
 static void
 prepare_diff(cset & included,
              app_state & app,
-             std::vector<utf8> args,
+             args_vector args,
              bool & new_is_archived,
              std::string & revheader)
 {
@@ -381,6 +381,8 @@ prepare_diff(cset & included,
       app.work.update_current_roster_from_filesystem(new_roster, mask);
       make_restricted_csets(old_roster, new_roster,
                             included, excluded, mask);
+      MM(included);
+      MM(excluded);
       check_restricted_cset(old_roster, included);
 
       new_is_archived = false;
@@ -406,6 +408,8 @@ prepare_diff(cset & included,
       app.work.update_current_roster_from_filesystem(new_roster, mask);
       make_restricted_csets(old_roster, new_roster,
                             included, excluded, mask);
+      MM(included);
+      MM(excluded);
       check_restricted_cset(old_roster, included);
 
       new_is_archived = false;
@@ -456,6 +460,8 @@ prepare_diff(cset & included,
 
       make_restricted_csets(old_roster, new_roster,
                             included, excluded, mask);
+      MM(included);
+      MM(excluded);
       check_restricted_cset(old_roster, included);
 
       new_is_archived = true;
@@ -468,11 +474,14 @@ prepare_diff(cset & included,
     revheader = header.str();
 }
 
-CMD(diff, N_("informative"), N_("[PATH]..."),
-    N_("show current diffs on stdout.\n"
-    "If one revision is given, the diff between the workspace and\n"
-    "that revision is shown.  If two revisions are given, the diff between\n"
-    "them is given.  If no format is specified, unified is used by default."),
+CMD(diff, "diff", "", CMD_REF(informative), N_("[PATH]..."),
+    N_("Shows current differences"),
+    N_("Compares the current tree with the files in the repository and "
+       "prints the differences on the standard output.\n"
+       "If one revision is given, the diff between the workspace and "
+       "that revision is shown.  If two revisions are given, the diff "
+       "between them is given.  If no format is specified, unified is "
+       "used by default."),
     options::opts::revision | options::opts::depth | options::opts::exclude
     | options::opts::diff_options)
 {
@@ -506,10 +515,14 @@ CMD(diff, N_("informative"), N_("[PATH]..."),
     }
   cout << "#\n";
 
-  if (app.opts.diff_format == external_diff) {
-    do_external_diff(included, app, new_is_archived);
-  } else
-    dump_diffs(included, app, new_is_archived, cout);
+  if (app.opts.diff_format == external_diff)
+    {
+      do_external_diff(included, app, new_is_archived);
+    }
+  else
+    {
+      dump_diffs(included, app, new_is_archived, cout);
+    }
 }
 
 
@@ -523,8 +536,11 @@ CMD(diff, N_("informative"), N_("[PATH]..."),
 // doubles the output of automate get_revision). If no content changes happened,
 // the output is empty. All file operations beside mtn add are omitted,
 // as they don't change the content of the file.
-AUTOMATE(content_diff, N_("[FILE [...]]"),
-    options::opts::revision | options::opts::depth | options::opts::exclude)
+CMD_AUTOMATE(content_diff, N_("[FILE [...]]"),
+             N_("Calculates diffs of files"),
+             "",
+             options::opts::revision | options::opts::depth |
+             options::opts::exclude)
 {
   cset included;
   std::string dummy_header;
@@ -597,9 +613,11 @@ typedef priority_queue<pair<rev_height, revision_id>,
                        vector<pair<rev_height, revision_id> >,
                        rev_cmp> frontier_t;
 
-CMD(log, N_("informative"), N_("[FILE] ..."),
-    N_("print history in reverse order (filtering by 'FILE'). If one or more\n"
-    "revisions are given, use them as a starting point."),
+CMD(log, "log", "", CMD_REF(informative), N_("[FILE] ..."),
+    N_("Prints history in reverse order"),
+    N_("This command prints history in reverse order, filtering it by "
+       "FILE if given.  If one or more revisions are given, uses them as "
+       "a starting point."),
     options::opts::last | options::opts::next
     | options::opts::from | options::opts::to
     | options::opts::brief | options::opts::diffs
@@ -632,7 +650,7 @@ CMD(log, N_("informative"), N_("[FILE] ..."),
     }
   else
     {
-      for (vector<utf8>::const_iterator i = app.opts.from.begin();
+      for (args_vector::const_iterator i = app.opts.from.begin();
            i != app.opts.from.end(); i++)
         {
           set<revision_id> rids;
@@ -686,7 +704,7 @@ CMD(log, N_("informative"), N_("[FILE] ..."),
   if (use_disallowed)
     {
       std::deque<revision_id> to;
-      for (vector<utf8>::const_iterator i = app.opts.to.begin();
+      for (args_vector::const_iterator i = app.opts.to.begin();
            i != app.opts.to.end(); i++)
         {
           MM(*i);
@@ -756,7 +774,7 @@ CMD(log, N_("informative"), N_("[FILE] ..."),
       revision_id const & rid = frontier.top().second;
 
       bool print_this = mask.empty();
-      set<split_path> diff_paths;
+      set<file_path> diff_paths;
 
       if (null_id(rid) || seen.find(rid) != seen.end())
         {
@@ -812,9 +830,9 @@ CMD(log, N_("informative"), N_("[FILE] ..."),
                       print_this = true;
                       if (app.opts.diffs)
                         {
-                          split_path sp;
-                          roster.get_name(*n, sp);
-                          diff_paths.insert(sp);
+                          file_path fp;
+                          roster.get_name(*n, fp);
+                          diff_paths.insert(fp);
                         }
                     }
                 }
