@@ -7,6 +7,7 @@
 // implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 // PURPOSE.
 
+#include "base.hh"
 #include <iostream>
 #include <sstream>
 #include <fstream>
@@ -29,11 +30,13 @@ using std::ofstream;
 using Botan::Pipe;
 using Botan::RSA_PrivateKey;
 
-CMD(genkey, N_("key and cert"), N_("KEYID"), N_("generate an RSA key-pair"),
+CMD(genkey, "genkey", "", CMD_REF(key_and_cert), N_("KEYID"),
+    N_("Generates an RSA key-pair"),
+    "",
     options::opts::none)
 {
   if (args.size() != 1)
-    throw usage(name);
+    throw usage(execid);
 
   rsa_keypair_id ident;
   internalize_rsa_keypair_id(idx(args, 0), ident);
@@ -55,13 +58,15 @@ CMD(genkey, N_("key and cert"), N_("KEYID"), N_("generate an RSA key-pair"),
   app.keys.put_key_pair(ident, kp);
 }
 
-CMD(dropkey, N_("key and cert"), N_("KEYID"),
-    N_("drop a public and private key"), options::opts::none)
+CMD(dropkey, "dropkey", "", CMD_REF(key_and_cert), N_("KEYID"),
+    N_("Drops a public and/or private key"),
+    "",
+    options::opts::none)
 {
   bool key_deleted = false;
 
   if (args.size() != 1)
-    throw usage(name);
+    throw usage(execid);
 
   rsa_keypair_id ident(idx(args, 0)());
   bool checked_db = false;
@@ -95,12 +100,13 @@ CMD(dropkey, N_("key and cert"), N_("KEYID"),
   N(key_deleted, fmt % idx(args, 0)());
 }
 
-CMD(passphrase, N_("key and cert"), N_("KEYID"),
-    N_("change passphrase of a private RSA key"),
+CMD(passphrase, "passphrase", "", CMD_REF(key_and_cert), N_("KEYID"),
+    N_("Changes the passphrase of a private RSA key"),
+    "",
     options::opts::none)
 {
   if (args.size() != 1)
-    throw usage(name);
+    throw usage(execid);
 
   rsa_keypair_id ident;
   internalize_rsa_keypair_id(idx(args, 0), ident);
@@ -116,13 +122,14 @@ CMD(passphrase, N_("key and cert"), N_("KEYID"),
   P(F("passphrase changed"));
 }
 
-CMD(ssh_agent_export, N_("key and cert"),
+CMD(ssh_agent_export, "ssh_agent_export", "", CMD_REF(key_and_cert),
     N_("[FILENAME]"),
-    N_("export your monotone key for use with ssh-agent"),
+    N_("Exports a private key for use with ssh-agent"),
+    "",
     options::opts::none)
 {
   if (args.size() > 1)
-    throw usage(name);
+    throw usage(execid);
 
   rsa_keypair_id id;
   keypair key;
@@ -156,12 +163,13 @@ CMD(ssh_agent_export, N_("key and cert"),
     }
 }
 
-CMD(ssh_agent_add, N_("key and cert"), "",
-    N_("Add your monotone key to ssh-agent"),
+CMD(ssh_agent_add, "ssh_agent_add", "", CMD_REF(key_and_cert), "",
+    N_("Adds a private key to ssh-agent"),
+    "",
     options::opts::none)
 {
   if (args.size() > 1)
-    throw usage(name);
+    throw usage(execid);
 
   rsa_keypair_id id;
   keypair key;
@@ -172,19 +180,22 @@ CMD(ssh_agent_add, N_("key and cert"), "",
   app.agent.add_identity(*priv, id());
 }
 
-CMD(cert, N_("key and cert"), N_("REVISION CERTNAME [CERTVAL]"),
-    N_("create a cert for a revision"), options::opts::none)
+CMD(cert, "cert", "", CMD_REF(key_and_cert),
+    N_("REVISION CERTNAME [CERTVAL]"),
+    N_("Creates a certificate for a revision"),
+    "",
+    options::opts::none)
 {
   if ((args.size() != 3) && (args.size() != 2))
-    throw usage(name);
+    throw usage(execid);
 
   transaction_guard guard(app.db);
 
   revision_id rid;
   complete(app, idx(args, 0)(), rid);
 
-  cert_name name;
-  internalize_cert_name(idx(args, 1), name);
+  cert_name cname;
+  internalize_cert_name(idx(args, 1), cname);
 
   rsa_keypair_id key;
   get_user_key(key, app);
@@ -199,25 +210,25 @@ CMD(cert, N_("key and cert"), N_("REVISION CERTNAME [CERTVAL]"),
       val = cert_value(dat());
     }
 
-  app.get_project().put_cert(rid, name, val);
+  app.get_project().put_cert(rid, cname, val);
   guard.commit();
 }
 
-CMD(trusted, N_("key and cert"), 
+CMD(trusted, "trusted", "", CMD_REF(key_and_cert), 
     N_("REVISION NAME VALUE SIGNER1 [SIGNER2 [...]]"),
-    N_("test whether a hypothetical cert would be trusted\n"
-       "by current settings"),
+    N_("Tests whether a hypothetical certificate would be trusted"),
+    N_("The current settings are used to run the test."),
     options::opts::none)
 {
   if (args.size() < 4)
-    throw usage(name);
+    throw usage(execid);
 
   revision_id rid;
   complete(app, idx(args, 0)(), rid, false);
   hexenc<id> ident(rid.inner());
 
-  cert_name name;
-  internalize_cert_name(idx(args, 1), name);
+  cert_name cname;
+  internalize_cert_name(idx(args, 1), cname);
 
   cert_value value(idx(args, 2)());
 
@@ -231,7 +242,7 @@ CMD(trusted, N_("key and cert"),
 
 
   bool trusted = app.lua.hook_get_revision_cert_trust(signers, ident,
-                                                      name, value);
+                                                      cname, value);
 
 
   ostringstream all_signers;
@@ -244,18 +255,20 @@ CMD(trusted, N_("key and cert"),
             "was signed by: %s\n"
             "it would be: %s")
     % ident
-    % name
+    % cname
     % value
     % all_signers.str()
     % (trusted ? _("trusted") : _("UNtrusted")))
     << '\n'; // final newline is kept out of the translation
 }
 
-CMD(tag, N_("review"), N_("REVISION TAGNAME"),
-    N_("put a symbolic tag cert on a revision"), options::opts::none)
+CMD(tag, "tag", "", CMD_REF(review), N_("REVISION TAGNAME"),
+    N_("Puts a symbolic tag certificate on a revision"),
+    "",
+    options::opts::none)
 {
   if (args.size() != 2)
-    throw usage(name);
+    throw usage(execid);
 
   revision_id r;
   complete(app, idx(args, 0)(), r);
@@ -263,11 +276,14 @@ CMD(tag, N_("review"), N_("REVISION TAGNAME"),
 }
 
 
-CMD(testresult, N_("review"), N_("ID (pass|fail|true|false|yes|no|1|0)"),
-    N_("note the results of running a test on a revision"), options::opts::none)
+CMD(testresult, "testresult", "", CMD_REF(review),
+    N_("ID (pass|fail|true|false|yes|no|1|0)"),
+    N_("Notes the results of running a test on a revision"),
+    "",
+    options::opts::none)
 {
   if (args.size() != 2)
-    throw usage(name);
+    throw usage(execid);
 
   revision_id r;
   complete(app, idx(args, 0)(), r);
@@ -275,12 +291,13 @@ CMD(testresult, N_("review"), N_("ID (pass|fail|true|false|yes|no|1|0)"),
 }
 
 
-CMD(approve, N_("review"), N_("REVISION"),
-    N_("approve of a particular revision"),
+CMD(approve, "approve", "", CMD_REF(review), N_("REVISION"),
+    N_("Approves a particular revision"),
+    "",
     options::opts::branch)
 {
   if (args.size() != 1)
-    throw usage(name);
+    throw usage(execid);
 
   revision_id r;
   complete(app, idx(args, 0)(), r);
@@ -289,11 +306,28 @@ CMD(approve, N_("review"), N_("REVISION"),
   app.get_project().put_revision_in_branch(r, app.opts.branchname);
 }
 
-CMD(comment, N_("review"), N_("REVISION [COMMENT]"),
-    N_("comment on a particular revision"), options::opts::none)
+CMD(suspend, "suspend", "", CMD_REF(review), N_("REVISION"),
+    N_("Suspends a particular revision"),
+    "",
+    options::opts::branch)
+{
+  if (args.size() != 1)
+    throw usage(execid);
+
+  revision_id r;
+  complete(app, idx(args, 0)(), r);
+  guess_branch(r, app);
+  N(app.opts.branchname() != "", F("need --branch argument to suspend"));
+  app.get_project().suspend_revision_in_branch(r, app.opts.branchname);
+}
+
+CMD(comment, "comment", "", CMD_REF(review), N_("REVISION [COMMENT]"),
+    N_("Comments on a particular revision"),
+    "",
+    options::opts::none)
 {
   if (args.size() != 1 && args.size() != 2)
-    throw usage(name);
+    throw usage(execid);
 
   utf8 comment;
   if (args.size() == 2)
