@@ -1,4 +1,5 @@
 
+#include "base.hh"
 #include "lua.hh"
 
 #include <signal.h>
@@ -162,7 +163,12 @@ LUAEXT(sleep, )
   return 1;
 }
 
-
+LUAEXT(get_pid, )
+{
+  pid_t pid = get_process_id();
+  lua_pushnumber(L, pid);
+  return 1;
+}
 
 // Local Variables:
 // mode: C++
