@@ -7,24 +7,17 @@
 // implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 // PURPOSE.
 
-#include "config.h"
 
+#include "base.hh"
 #include <iterator>
 #include <iostream>
 #include <fstream>
 #include <sstream>
 #include <locale.h>
-
 #include <stdlib.h>
 
-#include <boost/filesystem/convenience.hpp>
-#include <boost/filesystem/path.hpp>
-#include <boost/shared_ptr.hpp>
-
 #include "botan/botan.h"
-
 #include "i18n.h"
-
 #include "app_state.hh"
 #include "commands.hh"
 #include "sanity.hh"
@@ -37,6 +30,8 @@
 #include "paths.hh"
 #include "sha1.hh"
 #include "simplestring_xform.hh"
+#include "platform.hh"
+
 
 using std::cout;
 using std::cerr;
@@ -47,7 +42,6 @@ using std::set;
 using std::string;
 using std::vector;
 using std::ios_base;
-using boost::shared_ptr;
 
 // main option processing and exception handling code
 
@@ -118,35 +112,51 @@ void localize_monotone()
   if (!init)
     {
       setlocale(LC_ALL, "");
-      bindtextdomain(PACKAGE, LOCALEDIR);
+      bindtextdomain(PACKAGE, get_locale_dir().c_str());
       textdomain(PACKAGE);
       init = 1;
     }
 }
 
-// read command-line options and return the command name
-string read_options(options & opts, vector<string> args)
+option::concrete_option_set
+read_global_options(options & opts, args_vector & args)
 {
   option::concrete_option_set optset =
     options::opts::all_options().instantiate(&opts);
   optset.from_command_line(args);
+  
+  return optset;
+}
 
-  // consume the command, and perform completion if necessary
-  string cmd;
-  if (!opts.args.empty())
-    cmd = commands::complete_command(idx(opts.args, 0)());
-
-  // reparse options, now that we know what command-specific
-  // options are allowed.
-
-  options::options_type cmdopts = commands::command_options(opts.args);
-  optset.reset();
-
-  optset = (options::opts::globals() | cmdopts).instantiate(&opts);
-  optset.from_command_line(args, false);
+// read command-line options and return the command name
+commands::command_id read_options(option::concrete_option_set & optset, options & opts, args_vector & args)
+{
+  commands::command_id cmd;
 
   if (!opts.args.empty())
-    opts.args.erase(opts.args.begin());
+    {
+      // There are some arguments remaining in the command line.  Try first
+      // to see if they are a command.
+      cmd = commands::complete_command(opts.args);
+      I(!cmd.empty());
+
+      // Reparse options now that we know what command-specific options
+      // are allowed.
+      options::options_type cmdopts = commands::command_options(cmd);
+      optset.reset();
+      optset = (options::opts::globals() | cmdopts).instantiate(&opts);
+      optset.from_command_line(args, false);
+
+      // Remove the command name from the arguments.  Rember that the group
+      // is not taken into account.
+      I(opts.args.size() >= cmd.size() - 1);
+
+      for (args_vector::size_type i = 1; i < cmd.size(); i++)
+        {
+          I(cmd[i]().find(opts.args[0]()) == 0);
+          opts.args.erase(opts.args.begin());
+        }
+    }
 
   return cmd;
 }
@@ -179,31 +189,34 @@ cpp_main(int argc, char ** argv)
       save_initial_path();
       
       // decode all argv values into a UTF-8 array
-      vector<string> args;
+      args_vector args;
       for (int i = 1; i < argc; ++i)
         {
           external ex(argv[i]);
           utf8 ut;
           system_to_utf8(ex, ut);
-          args.push_back(ut());
+          args.push_back(arg_type(ut));
         }
 
       // find base name of executable, convert to utf8, and save it in the
       // global ui object
       {
-        string prog_name = fs::path(argv[0]).leaf();
+        utf8 argv0_u;
+        system_to_utf8(external(argv[0]), argv0_u);
+        string prog_name = system_path(argv0_u).basename()();
         if (prog_name.rfind(".exe") == prog_name.size() - 4)
           prog_name = prog_name.substr(0, prog_name.size() - 4);
-        utf8 prog_name_u;
-        system_to_utf8(external(prog_name), prog_name_u);
-        ui.prog_name = prog_name_u();
+        ui.prog_name = prog_name;
         I(!ui.prog_name.empty());
       }
 
       app_state app;
       try
         {
-          string cmd = read_options(app.opts, args);
+          // read global options first
+          // command specific options will be read below
+          args_vector opt_args(args);
+          option::concrete_option_set optset = read_global_options(app.opts, opt_args);
 
           if (app.opts.version_given)
             {
@@ -211,30 +224,18 @@ cpp_main(int argc, char ** argv)
               return 0;
             }
 
-          if (app.opts.full_version_given)
-            {
-              print_full_version();
-              return 0;
-            }
-
           if (app.opts.dbname_given)
             {
               if (!app.opts.dbname.empty())
                 app.db.set_filename(app.opts.dbname);
             }
 
-          if (app.opts.key_dir_given)
+          if (app.opts.key_dir_given || app.opts.conf_dir_given)
             {
               if (!app.opts.key_dir.empty())
                 app.keys.set_key_dir(app.opts.key_dir);
             }
 
-          // stop here if they asked for help
-          if (app.opts.help)
-            {
-              throw usage(cmd);     // cmd may be empty, and that's fine.
-            }
-
           // at this point we allow a workspace (meaning search for it
           // and if found read _MTN/options, but don't use the data quite
           // yet, and read all the monotonercs).  Processing the data
@@ -243,21 +244,35 @@ cpp_main(int argc, char ** argv)
           // if we didn't find one at this point.
           app.allow_workspace();
 
-          if (!app.found_workspace && global_sanity.filename.empty())
-            global_sanity.filename = (app.opts.conf_dir / "dump").as_external();
+          // now grab any command specific options and parse the command
+          // this needs to happen after the monotonercs have been read
+          commands::command_id cmd = read_options(optset, app.opts, opt_args);
+
+          if (!app.found_workspace)
+            global_sanity.set_dump_path((app.opts.conf_dir / "dump")
+                                        .as_external());
 
           app.lua.hook_note_mtn_startup(args);
 
+          // stop here if they asked for help
+          if (app.opts.help)
+            {
+              throw usage(cmd);
+            }
+
           // main options processed, now invoke the
           // sub-command w/ remaining args
           if (cmd.empty())
             {
-              throw usage("");
+              throw usage(commands::command_id());
             }
           else
             {
-              vector<utf8> args(app.opts.args.begin(), app.opts.args.end());
-              return commands::process(app, cmd, args);
+              commands::process(app, cmd, app.opts.args);
+              // The command will raise any problems itself through
+              // exceptions.  If we reach this point, it is because it
+              // worked correctly.
+              return 0;
             }
         }
       catch (option::option_error const & e)
@@ -272,16 +287,26 @@ cpp_main(int argc, char ** argv)
           // merrily down your pipes.
           std::ostream & usage_stream = (app.opts.help ? cout : cerr);
 
-          usage_stream << F("Usage: %s [OPTION...] command [ARG...]") % ui.prog_name << "\n\n";
-          usage_stream << options::opts::globals().instantiate(&app.opts).get_usage_str() << '\n';
+          string visibleid;
+          if (!u.which.empty())
+            visibleid = join_words(vector< utf8 >(u.which.begin() + 1,
+                                                  u.which.end()))();
+
+          usage_stream << F("Usage: %s [OPTION...] command [ARG...]") %
+                          ui.prog_name << "\n\n";
+          usage_stream << options::opts::globals().instantiate(&app.opts).
+                          get_usage_str() << '\n';
 
           // Make sure to hide documentation that's not part of
           // the current command.
-          options::options_type cmd_options = commands::toplevel_command_options(u.which);
+          options::options_type cmd_options =
+            commands::command_options(u.which);
           if (!cmd_options.empty())
             {
-              usage_stream << F("Options specific to '%s %s':") % ui.prog_name % u.which << "\n\n";
-              usage_stream << cmd_options.instantiate(&app.opts).get_usage_str() << '\n';
+              usage_stream << F("Options specific to '%s %s':") %
+                              ui.prog_name % visibleid << "\n\n";
+              usage_stream << cmd_options.instantiate(&app.opts).
+                              get_usage_str() << '\n';
             }
 
           commands::explain_usage(u.which, usage_stream);
