@@ -11,14 +11,16 @@
 // the only .cc file that depends on the revision/full_revision header files,
 // which change constantly.
 
-#include "config.h"
 
+#include "base.hh"
 #include <iostream>
 #include <sstream>
 
 #include <boost/version.hpp>
 #include <boost/config.hpp>
 
+#include "app_state.hh"
+#include "cmd.hh"
 #include "platform.hh"
 #include "mt_version.hh"
 #include "package_revision.h"
@@ -29,6 +31,22 @@ using std::cout;
 using std::ostringstream;
 using std::string;
 
+CMD(version, "version", "", CMD_REF(informative), "",
+    N_("Shows the program version"),
+    "",
+    options::opts::full)
+{
+  N(args.size() == 0,
+    F("no arguments allowed"));
+
+  string version;
+  if (app.opts.full)
+    get_full_version(version);
+  else
+    get_version(version);
+  cout << version << '\n';
+}
+
 void
 get_version(string & out)
 {
