@@ -16,8 +16,12 @@ REV3=base_revision()
 
 testURI="file:" .. test.root .. "/test.db"
 
-check(nodb_mtn("--branch=testbranch", "clone", testURI, "test_dir1"),
+check(nodb_mtn("--branch=testbranch", "clone", testURI, "test_dirA"),
          1, false, true)
-xfail_if(ostype == "Windows", not exists("test_dir1"))
 check(qgrep(REV2, "stderr"))
 check(qgrep(REV3, "stderr"))
+
+on_windows=(ostype == "Windows")
+on_solaris=(string.find(ostype, 'SunOS') ~= nil)
+bad_platform=(on_solaris or on_windows)
+xfail_if(bad_platform, not exists("test_dirA"))
