@@ -16,7 +16,7 @@ check(mtn("cert", rev, "author", "extra_author"), 0, false, false)
 -- if we drop the file, we'll have a roster that doesn't
 -- reference its own revision. 
 -- we can then remove the revision to end up with a clean unreferenced roster.
-check(mtn("drop", "testfile"), 0, false, false)
+check(mtn("drop", "--bookkeep-only", "testfile"), 0, false, false)
 check(mtn("commit", "-m", "goingaway"), 0, false, false)
 del_rev = base_revision()
 for a,b in pairs({revisions = "id", revision_certs = "id", revision_ancestry = "child"}) do
