@@ -15,7 +15,7 @@ mtn_setup()
 -- without the invariant, it would silently throw such revisions away.)
 
 remove("test.db")
-get("test.db.dumped", "stdin")
+check(get("test.db.dumped", "stdin"))
 check(mtn("db", "load"), 0, false, false, true)
 check(mtn("db", "migrate"), 0, false, false)
 
@@ -29,6 +29,6 @@ for l in io.lines("stdout") do
   table.insert(revs, l)
 end
 
-get("manifest_good")
+check(get("manifest_good"))
 check(mtn("automate", "get_manifest_of", revs[1]), 0, true)
 check(samefile("stdout", "manifest_good"))
