@@ -6,13 +6,13 @@ addfile("bar", "file bar\n")
 commit("branch")
 base = base_revision()
 
-check(mtn("mv", "-e", "foo", "baz"), 0, false, false)
+check(mtn("mv", "foo", "baz"), 0, false, false)
 append("bar", "xxx\n")
 commit("branch")
 left = base_revision()
 
 check(mtn("update", "-r", base), 0, false, false)
-check(mtn("mv", "-e", "foo", "quux"), 0, false, false)
+check(mtn("mv", "foo", "quux"), 0, false, false)
 append("bar", "yyy\n")
 commit("branch")
 right = base_revision()
