@@ -16,6 +16,10 @@ check(samefile("stdout", "orig_rev"))
 -- running a recursive add what's supposed to be a file, but is actually a
 -- dir...
 mkdir("dir2")
-check(mtn("rename", "--bookkeep-only", "file", "dir2"), 1, false, false)
-check(mtn("add", "dir2"), 0, false, false)
-check(mtn("rename", "file", "dir2"), 0, false, false)
+check(mtn("rename", "--bookkeep-only", "file", "dir2"), 0, false, false)
+-- should now be already added (this is a soft error now.  see stderr to
+-- ensure proper response
+check(mtn("add", "dir2"), 0, false, true)
+check(qgrep("skipping dir2, already accounted for in workspace", "stderr"))
+-- should have happened already in the --bookkeep-only version above.
+check(mtn("rename", "file", "dir2"), 1, false, false)
