@@ -7,8 +7,8 @@
 // implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 // PURPOSE.
 
-#include <string>
 
+#include "base.hh"
 #include "constants.hh"
 #include "hash_map.hh"
 #include "sanity.hh"
@@ -49,13 +49,6 @@ static inline void
 verify_full(T & val)
 { val.ok = true; }
 
-inline void
-verify_full(path_component & val)
-{
-  // FIXME: probably ought to do something here?
-  val.ok = true;
-}
-
 // NOTE: _not_ verify_full; you use verify_full for ATOMICs, verify() for
 // everything else.
 inline void
@@ -77,16 +70,6 @@ verify(hexenc<id> & val)
   val.ok = true;
 }
 
-inline void
-verify_full(ace & val)
-{
-  string::size_type pos = val().find_first_not_of(constants::legal_ace_bytes);
-  N(pos == string::npos,
-    F("bad character '%c' in ace string '%s'") % val().at(pos) % val);
-
-  val.ok = true;
-}
-
 inline void
 verify_full(symbol & val)
 {
@@ -124,7 +107,7 @@ verify_full(netsync_session_key & val)
 {
   if (val().size() == 0)
     {
-      val.s.append(constants::netsync_session_key_length_in_bytes, 0);
+      val.s = std::string(constants::netsync_session_key_length_in_bytes, 0);
       return;
     }
 
@@ -139,7 +122,7 @@ verify_full(netsync_hmac_value & val)
 {
   if (val().size() == 0)
     {
-      val.s.append(constants::netsync_hmac_value_length_in_bytes, 0);
+      val.s = std::string(constants::netsync_hmac_value_length_in_bytes, 0);
       return;
     }
 
@@ -237,53 +220,53 @@ void dump(manifest_data const & d, string &);
 UNIT_TEST(vocab, verify_hexenc_id)
 {
   // -------- magic empty string and default constructor are okay:
-  BOOST_CHECK(hexenc<id>("")() == "");
+  UNIT_TEST_CHECK(hexenc<id>("")() == "");
   hexenc<id> my_default_id;
-  BOOST_CHECK(my_default_id() == "");
+  UNIT_TEST_CHECK(my_default_id() == "");
 
   // -------- wrong length:
-  BOOST_CHECK_THROW(hexenc<id>("a"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("a"), informative_failure);
   // 39 letters
-  BOOST_CHECK_THROW(hexenc<id>("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"),
+  UNIT_TEST_CHECK_THROW(hexenc<id>("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"),
                     informative_failure);
   // 41 letters
-  BOOST_CHECK_THROW(hexenc<id>("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"),
+  UNIT_TEST_CHECK_THROW(hexenc<id>("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"),
                     informative_failure);
   // but 40 is okay
-  BOOST_CHECK(hexenc<id>("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")()
+  UNIT_TEST_CHECK(hexenc<id>("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")()
               == "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa");
 
   // -------- bad characters:
-  BOOST_CHECK_THROW(hexenc<id>("g000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("h000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("G000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("H000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("*000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("`000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("z000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("Z000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("g000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("h000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("G000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("H000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("*000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("`000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("z000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("Z000000000000000000000000000000000000000"), informative_failure);
   // different positions:
-  BOOST_CHECK_THROW(hexenc<id>("g000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("0g00000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("00g0000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("000g000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("0000g00000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("000000000000000000000g000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("0000000000000000000000g00000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("000000000000000000000000000000g000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("000000000000000000000000000000000000g000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("0000000000000000000000000000000000000g00"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("00000000000000000000000000000000000000g0"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("000000000000000000000000000000000000000g"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("g000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("0g00000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("00g0000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("000g000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("0000g00000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("000000000000000000000g000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("0000000000000000000000g00000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("000000000000000000000000000000g000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("000000000000000000000000000000000000g000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("0000000000000000000000000000000000000g00"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("00000000000000000000000000000000000000g0"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("000000000000000000000000000000000000000g"), informative_failure);
   // uppercase hex is bad too!
-  BOOST_CHECK_THROW(hexenc<id>("A000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("B000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("C000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("D000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("E000000000000000000000000000000000000000"), informative_failure);
-  BOOST_CHECK_THROW(hexenc<id>("F000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("A000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("B000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("C000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("D000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("E000000000000000000000000000000000000000"), informative_failure);
+  UNIT_TEST_CHECK_THROW(hexenc<id>("F000000000000000000000000000000000000000"), informative_failure);
   // but lowercase and digits are all fine
-  BOOST_CHECK(hexenc<id>("0123456789abcdef0123456789abcdef01234567")()
+  UNIT_TEST_CHECK(hexenc<id>("0123456789abcdef0123456789abcdef01234567")()
               == "0123456789abcdef0123456789abcdef01234567");
 }
 
