@@ -4,10 +4,10 @@
 // licensed to the public under the terms of the GNU GPL (>= 2)
 // see the file COPYING for details
 
+#include "base.hh"
 #include <windows.h>
 #include <io.h>
 
-#include <string>
 
 #include "platform.hh"
 
